@@ -3,6 +3,7 @@
 
 #include "avif/internal.h"
 
+#include <assert.h>
 #include <math.h>
 #include <string.h>
 
@@ -15,45 +16,40 @@ float avifRoundf(float v)
 
 uint16_t avifHTONS(uint16_t s)
 {
-    uint8_t data[2];
+    uint16_t result;
+    uint8_t * data = (uint8_t *)&result;
     data[0] = (s >> 8) & 0xff;
     data[1] = (s >> 0) & 0xff;
-    uint16_t result;
-    memcpy(&result, data, sizeof(uint16_t));
     return result;
 }
 
 uint16_t avifNTOHS(uint16_t s)
 {
-    uint8_t data[2];
-    memcpy(&data, &s, sizeof(data));
-
+    const uint8_t * data = (const uint8_t *)&s;
     return (uint16_t)((data[1] << 0) | (data[0] << 8));
 }
 
 uint32_t avifHTONL(uint32_t l)
 {
-    uint8_t data[4];
+    uint32_t result;
+    uint8_t * data = (uint8_t *)&result;
     data[0] = (l >> 24) & 0xff;
     data[1] = (l >> 16) & 0xff;
     data[2] = (l >> 8) & 0xff;
     data[3] = (l >> 0) & 0xff;
-    uint32_t result;
-    memcpy(&result, data, sizeof(uint32_t));
     return result;
 }
 
 uint32_t avifNTOHL(uint32_t l)
 {
-    uint8_t data[4];
-    memcpy(&data, &l, sizeof(data));
-
+    const uint8_t * data = (const uint8_t *)&l;
     return ((uint32_t)data[3] << 0) | ((uint32_t)data[2] << 8) | ((uint32_t)data[1] << 16) | ((uint32_t)data[0] << 24);
 }
 
 uint64_t avifHTON64(uint64_t l)
 {
-    uint8_t data[8];
+    uint64_t result;
+    uint8_t * data = (uint8_t *)&result;
     data[0] = (l >> 56) & 0xff;
     data[1] = (l >> 48) & 0xff;
     data[2] = (l >> 40) & 0xff;
@@ -62,23 +58,20 @@ uint64_t avifHTON64(uint64_t l)
     data[5] = (l >> 16) & 0xff;
     data[6] = (l >> 8) & 0xff;
     data[7] = (l >> 0) & 0xff;
-    uint64_t result;
-    memcpy(&result, data, sizeof(uint64_t));
     return result;
 }
 
 uint64_t avifNTOH64(uint64_t l)
 {
-    uint8_t data[8];
-    memcpy(&data, &l, sizeof(data));
-
+    const uint8_t * data = (const uint8_t *)&l;
     return ((uint64_t)data[7] << 0) | ((uint64_t)data[6] << 8) | ((uint64_t)data[5] << 16) | ((uint64_t)data[4] << 24) |
            ((uint64_t)data[3] << 32) | ((uint64_t)data[2] << 40) | ((uint64_t)data[1] << 48) | ((uint64_t)data[0] << 56);
 }
 
 AVIF_ARRAY_DECLARE(avifArrayInternal, uint8_t, ptr);
 
-void avifArrayCreate(void * arrayStruct, uint32_t elementSize, uint32_t initialCapacity)
+// On error, this function must set arr->ptr to NULL and both arr->count and arr->capacity to 0.
+avifBool avifArrayCreate(void * arrayStruct, uint32_t elementSize, uint32_t initialCapacity)
 {
     avifArrayInternal * arr = (avifArrayInternal *)arrayStruct;
     arr->elementSize = elementSize ? elementSize : 1;
@@ -86,7 +79,12 @@ void avifArrayCreate(void * arrayStruct, uint32_t elementSize, uint32_t initialC
     arr->capacity = initialCapacity;
     size_t byteCount = (size_t)arr->elementSize * arr->capacity;
     arr->ptr = (uint8_t *)avifAlloc(byteCount);
+    if (!arr->ptr) {
+        arr->capacity = 0;
+        return AVIF_FALSE;
+    }
     memset(arr->ptr, 0, byteCount);
+    return AVIF_TRUE;
 }
 
 uint32_t avifArrayPushIndex(void * arrayStruct)
@@ -109,7 +107,7 @@ void * avifArrayPushPtr(void * arrayStruct)
 {
     uint32_t index = avifArrayPushIndex(arrayStruct);
     avifArrayInternal * arr = (avifArrayInternal *)arrayStruct;
-    return &arr->ptr[index * arr->elementSize];
+    return &arr->ptr[index * (size_t)arr->elementSize];
 }
 
 void avifArrayPush(void * arrayStruct, void * element)
@@ -119,6 +117,14 @@ void avifArrayPush(void * arrayStruct, void * element)
     memcpy(newElement, element, arr->elementSize);
 }
 
+void avifArrayPop(void * arrayStruct)
+{
+    avifArrayInternal * arr = (avifArrayInternal *)arrayStruct;
+    assert(arr->count > 0);
+    --arr->count;
+    memset(&arr->ptr[arr->count * (size_t)arr->elementSize], 0, arr->elementSize);
+}
+
 void avifArrayDestroy(void * arrayStruct)
 {
     avifArrayInternal * arr = (avifArrayInternal *)arrayStruct;
